#!/usr/bin/env python3
#
# Simple test suite

import os
import sys
import unittest

sys.path.insert(0, os.path.abspath(os.path.dirname(__file__) + '/../'))

from ffmpeg_quality_metrics import FfmpegQualityMetrics as ffqm

DIST = os.path.join(os.path.dirname(__file__), "dist-854x480.mkv")
REF = os.path.join(os.path.dirname(__file__), "ref-1280x720.mkv")


class TestMetrics(unittest.TestCase):
    def test_ssim(self):
        run_ret = ffqm(REF, DIST).calc_ssim_psnr()["ssim"]
        expected = [{'n': 1, 'ssim_y': 0.934, 'ssim_u': 0.96, 'ssim_v': 0.942, 'ssim_avg': 0.945}, {'n': 2, 'ssim_y': 0.934, 'ssim_u': 0.96, 'ssim_v': 0.943, 'ssim_avg': 0.946}, {'n': 3, 'ssim_y': 0.934, 'ssim_u': 0.959, 'ssim_v': 0.943, 'ssim_avg': 0.945}]
        for expected_frame, actual_frame in zip(expected, run_ret):
            for key in ['ssim_y', 'ssim_u', 'ssim_v']:
                self.assertAlmostEqual(expected_frame[key], actual_frame[key], places=2)

    def test_psnr(self):
        run_ret = ffqm(REF, DIST).calc_ssim_psnr()["psnr"]
        expected = [{'n': 1, 'mse_avg': 536.71, 'mse_y': 900.22, 'mse_u': 234.48, 'mse_v': 475.43, 'psnr_avg': 20.83, 'psnr_y': 18.59, 'psnr_u': 24.43, 'psnr_v': 21.36}, {'n': 2, 'mse_avg': 535.29, 'mse_y': 896.98, 'mse_u': 239.4, 'mse_v': 469.49, 'psnr_avg': 20.84, 'psnr_y': 18.6, 'psnr_u': 24.34, 'psnr_v': 21.41}, {'n': 3, 'mse_avg': 535.04, 'mse_y': 894.89, 'mse_u': 245.8, 'mse_v': 464.43, 'psnr_avg': 20.85, 'psnr_y': 18.61, 'psnr_u': 24.22, 'psnr_v': 21.46}]
        for expected_frame, actual_frame in zip(expected, run_ret):
            for key in ['mse_avg', 'mse_y', 'mse_u', 'mse_v', 'psnr_avg', 'psnr_y', 'psnr_u', 'psnr_v']:
                self.assertAlmostEqual(expected_frame[key], actual_frame[key], places=2)

    def test_vmaf(self):
        run_ret = ffqm(REF, DIST).calc_vmaf()
        expected = [{'psnr': 18.587308, 'integer_motion2': 0.0, 'integer_motion': 0.0, 'ssim': 0.925976, 'integer_adm2': 0.69907, 'integer_adm_scale0': 0.708183, 'integer_adm_scale1': 0.733469, 'integer_adm_scale2': 0.718624, 'integer_adm_scale3': 0.67301, 'integer_vif_scale0': 0.539591, 'integer_vif_scale1': 0.718022, 'integer_vif_scale2': 0.751875, 'integer_vif_scale3': 0.773503, 'ms_ssim': 0.898265, 'vmaf': 14.054853, 'n': 1}, {'psnr': 18.60299, 'integer_motion2': 0.359752, 'integer_motion': 0.368929, 'ssim': 0.926521, 'integer_adm2': 0.698451, 'integer_adm_scale0': 0.706706, 'integer_adm_scale1': 0.73203, 'integer_adm_scale2': 0.718262, 'integer_adm_scale3': 0.672766, 'integer_vif_scale0': 0.540231, 'integer_vif_scale1': 0.719566, 'integer_vif_scale2': 0.753567, 'integer_vif_scale3': 0.775864, 'ms_ssim': 0.899353, 'vmaf': 14.464182, 'n': 2}, {'psnr': 18.613101, 'integer_motion2': 0.359752, 'integer_motion': 0.359752, 'ssim': 0.926481, 'integer_adm2': 0.697126, 'integer_adm_scale0': 0.706542, 'integer_adm_scale1': 0.731351, 'integer_adm_scale2': 0.716454, 'integer_adm_scale3': 0.671197, 'integer_vif_scale0': 0.539091, 'integer_vif_scale1': 0.718657, 'integer_vif_scale2': 0.753306, 'integer_vif_scale3': 0.775984, 'ms_ssim': 0.900086, 'vmaf': 14.256442, 'n': 3}]
        for expected_frame, actual_frame in zip(expected, run_ret):
            for key in expected_frame.keys():
                self.assertAlmostEqual(expected_frame[key], actual_frame[key], places=2)

    def test_global(self):
        f = ffqm(REF, DIST)
        f.calc_ssim_psnr()
        run_ret = f.get_global_stats()
        expected = {'ssim': {'average': 0.945, 'median': 0.945, 'stdev': 0.0, 'min': 0.945, 'max': 0.946}, 'psnr': {'average': 20.84, 'median': 20.84, 'stdev': 0.008, 'min': 20.83, 'max': 20.85}}
        self.assertEqual(run_ret, expected)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python3
#
# ffmpeg-quality-metrics
# Author: Werner Robitza
# License: MIT

import argparse
import sys

from .ffmpeg_quality_metrics import FfmpegQualityMetrics
from .utils import print_error, print_warning

from .__init__ import __version__ as version


def main():
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
        description="ffmpeg_quality_metrics v" + version,
    )
    parser.add_argument("dist", help="input file, distorted")
    parser.add_argument("ref", help="input file, reference")

    general_opts = parser.add_argument_group("General options")

    general_opts.add_argument(
        "-n",
        "--dry-run",
        action="store_true",
        help="Do not run commands, just show what would be done",
    )
    general_opts.add_argument(
        "-v", "--verbose", action="store_true", help="Show verbose output"
    )
    general_opts.add_argument(
        "-pr", "--progress", action="store_true", help="Show a progress bar"
    )

    ffmpeg_opts = parser.add_argument_group("FFmpeg options")

    ffmpeg_opts.add_argument(
        "-dp",
        "--disable-psnr-ssim",
        action="store_true",
        help="Disable PSNR/SSIM computation. Use VMAF to get YUV estimate.",
    )

    ffmpeg_opts.add_argument(
        "-s",
        "--scaling-algorithm",
        default="bicubic",
        choices=FfmpegQualityMetrics.ALLOWED_SCALERS,
        help="Scaling algorithm for ffmpeg",
    )

    ffmpeg_opts.add_argument(
        "-r",
        "--framerate",
        type=float,
        help="Force an input framerate",
    )

    ffmpeg_opts.add_argument(
        "-t",
        "--threads",
        type=int,
        default=FfmpegQualityMetrics.DEFAULT_THREADS,
        help="Number of threads to do the calculations",
    )

    output_opts = parser.add_argument_group("Output options")

    output_opts.add_argument(
        "-of",
        "--output-format",
        type=str,
        default="json",
        choices=["json", "csv"],
        help="Output format for the metrics",
    )

    vmaf_opts = parser.add_argument_group("VMAF options")

    vmaf_opts.add_argument(
        "-ev",
        "--enable-vmaf",
        action="store_true",
        help="Enable VMAF computation; calculates VMAF as well as SSIM and PSNR",
    )

    vmaf_opts.add_argument(
        "-m",
        "--model-path",
        type=str,
        default=FfmpegQualityMetrics.get_default_vmaf_model_path(),
        help="Use a specific VMAF model file. If none is chosen, picks a default model. "
        f"You can also specify one of the following built-in models: {FfmpegQualityMetrics.get_supplied_vmaf_models()}",
    )

    vmaf_opts.add_argument(
        "-p", "--phone-model", action="store_true", help="Enable VMAF phone model"
    )

    vmaf_opts.add_argument(
        "-nt",
        "--n-threads",
        type=int,
        default=FfmpegQualityMetrics.DEFAULT_VMAF_THREADS,
        help="Set the value of libvmaf's n_threads option. "
        "This determines the number of threads that are used for VMAF calculation",
    )

    cli_args = parser.parse_args()

    ffqm = FfmpegQualityMetrics(
        ref=cli_args.ref,
        dist=cli_args.dist,
        scaling_algorithm=cli_args.scaling_algorithm,
        framerate=cli_args.framerate,
        dry_run=cli_args.dry_run,
        verbose=cli_args.verbose,
        threads=cli_args.threads,
        progress=cli_args.progress,
    )

    if cli_args.enable_vmaf:
        ffqm.calc_vmaf(
            model_path=cli_args.model_path,
            phone_model=cli_args.phone_model,
            n_threads=cli_args.n_threads,
        )

    if not cli_args.disable_psnr_ssim:
        ffqm.calc_ssim_psnr()

    if cli_args.dry_run:
        print_warning("Dry run specified, exiting without computing stats")
        return

    if cli_args.output_format == "json":
        print(ffqm.get_results_json())
    elif cli_args.output_format == "csv":
        print(ffqm.get_results_csv())
    else:
        print_error("Wrong output format chosen, use 'json' or 'csv'")
        sys.exit(1)


if __name__ == "__main__":
    try:
        main()
    except Exception as e:
        print_error(f"General exception: {e}")
        sys.exit(1)



# ffmpeg-quality-metrics
# Author: Werner Robitza
# License: MIT

from platform import system as _current_os
from shutil import which
import os
import sys
import subprocess
import shlex

CUR_OS = _current_os()
IS_WIN = CUR_OS in ["Windows", "cli"]
IS_NIX = (not IS_WIN) and any(
    CUR_OS.startswith(i)
    for i in ["CYGWIN", "MSYS", "Linux", "Darwin", "SunOS", "FreeBSD", "NetBSD"]
)
NUL = "NUL" if IS_WIN else "/dev/null"


def win_path_check(path):
    """
    Format a file path correctly for Windows
    """
    if IS_WIN:
        return path.replace("\\", "/").replace(":", "\\:")
    return path


def has_brew():
    """
    Check if the user has Homebrew installed
    """
    return which("brew") is not None


def ffmpeg_is_from_brew():
    """
    Is the used ffmpeg from Homebrew?
    """
    ffmpeg_path = which("ffmpeg")
    if ffmpeg_path is None:
        return False

    return os.path.islink(ffmpeg_path) and "Cellar/ffmpeg" in os.readlink(ffmpeg_path)


def print_error(msg):
    print("ERROR: %s" % msg, file=sys.stderr)


def print_warning(msg):
    print("WARNING: %s" % msg, file=sys.stderr)


def print_info(msg):
    print("INFO: %s" % msg, file=sys.stderr)


def quoted_cmd(cmd):
    return " ".join([shlex.quote(c) for c in cmd])


def run_command(cmd, dry_run=False, verbose=False, allow_error=False):
    """
    Run a command directly
    """
    if dry_run or verbose:
        print_info(quoted_cmd(cmd))
        if dry_run:
            return

    process = subprocess.Popen(cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    stdout, stderr = process.communicate()

    if allow_error or process.returncode == 0:
        return stdout.decode("utf-8"), stderr.decode("utf-8")
    else:
        raise RuntimeError(
            f"error running command: {quoted_cmd(cmd)}\n{stderr.decode('utf-8')}"
        )



# ffmpeg-quality-metrics
# Author: Werner Robitza
# License: MIT

from functools import reduce
import os
import json
import tempfile
import numpy as np
import re
import pandas as pd
from tqdm import tqdm
from ffmpeg_progress_yield import FfmpegProgress

from .utils import (
    NUL,
    ffmpeg_is_from_brew,
    has_brew,
    print_info,
    print_warning,
    run_command,
    win_path_check,
)


class FfmpegQualityMetricsError(Exception):
    pass


class FfmpegQualityMetrics:
    """
    A class to calculate quality metrics with FFmpeg
    """

    ALLOWED_SCALERS = [
        "fast_bilinear",
        "bilinear",
        "bicubic",
        "experimental",
        "neighbor",
        "area",
        "bicublin",
        "gauss",
        "sinc",
        "lanczos",
        "spline",
    ]
    DEFAULT_SCALER = "bicubic"
    DEFAULT_THREADS = 0
    DEFAULT_VMAF_THREADS = os.cpu_count()
    DEFAULT_VMAF_MODEL_DIRECTORY = os.path.join(
        os.path.dirname(__file__), "vmaf_models"
    )

    def __init__(
        self,
        ref: str,
        dist: str,
        scaling_algorithm=DEFAULT_SCALER,
        framerate=None,
        dry_run=False,
        verbose=False,
        threads=DEFAULT_THREADS,
        progress=False,
    ):
        """Instantiate a new FfmpegQualityMetrics

        Args:
            ref (str): reference file
            dist (str): distorted file
            scaling_algorithm (str, optional): A scaling algorithm. Defaults to "bicubic".
            framerate (float, optional): Force a frame rate. Defaults to None.
            dry_run (bool, optional): Don't run anything, just print commands. Defaults to False.
            verbose (bool, optional): Show more output. Defaults to False.
            threads (int, optional): Number of ffmpeg threads. Defaults to 0 (auto).
            progress (bool, optional): Show a progress bar. Defaults to False.

        Raises:
            FfmpegQualityMetricsError: A generic error
        """
        self.ref = str(ref)
        self.dist = str(dist)
        self.scaling_algorithm = str(scaling_algorithm)
        self.framerate = float(framerate) if framerate is not None else None
        self.dry_run = bool(dry_run)
        self.verbose = bool(verbose)
        self.threads = int(threads)
        self.progress = bool(progress)

        self.vmaf_data = []
        self.psnr_data = []
        self.ssim_data = []

        self.global_stats = {}

        self.temp_dir = tempfile.gettempdir()
        self.temp_files = {}
        for key in ["psnr", "ssim", "vmaf"]:
            self.temp_files[key] = os.path.join(
                self.temp_dir, next(tempfile._get_candidate_names()) + f"-{key}.txt"
            )

        if scaling_algorithm not in self.ALLOWED_SCALERS:
            raise FfmpegQualityMetricsError(
                f"Allowed scaling algorithms: {self.ALLOWED_SCALERS}"
            )

    @staticmethod
    def get_framerate(input_file):
        """Parse the FPS from the input file.

        Args:
            input_file (str): Input file path

        Raises:
            FfmpegQualityMetricsError: A generic error

        Returns:
            float: The FPS parsed
        """
        cmd = ["ffmpeg", "-nostdin", "-y", "-i", input_file]

        output = run_command(cmd, allow_error=True)
        pattern = re.compile(r"(\d+(\.\d+)?) fps")
        try:
            match = pattern.search(str(output)).groups()[0]
            return float(match)
        except Exception:
            raise FfmpegQualityMetricsError(
                f"could not parse FPS from file {input_file}!"
            )

    def _get_framerates(self):
        ref_framerate = FfmpegQualityMetrics.get_framerate(self.ref)
        dist_framerate = FfmpegQualityMetrics.get_framerate(self.dist)

        if ref_framerate != dist_framerate:
            print_warning(
                f"ref, dist framerates differ: {ref_framerate}, {dist_framerate}. "
                "This may result in inaccurate quality metrics. Force an input framerate via the -r option."
            )

        return ref_framerate, dist_framerate

    def calc_vmaf(
        self,
        model_path=None,
        phone_model=False,
        n_threads=os.cpu_count(),
    ):
        """Calculate the VMAF scores for the input files

        Args:
            model_path (str, optional): Path to the VMAF model. Defaults to 0.6.1 model.
            phone_model (bool, optional): Use phone model. Defaults to False.
            n_threads (int, optional): Number of VMAF threads. Defaults to os.cpu_count().

        Raises:
            FfmpegQualityMetricsError: A generic error

        Returns:
            dict: VMAF results
        """
        self.phone_model = bool(phone_model)

        if model_path is None:
            self.model_path = FfmpegQualityMetrics.get_default_vmaf_model_path()
        else:
            self.model_path = str(model_path)
        self.n_threads = int(n_threads)

        supplied_models = FfmpegQualityMetrics.get_supplied_vmaf_models()

        if not os.path.isfile(self.model_path):
            # check if this is one of the supplied ones? e.g. user passed only a filename
            if self.model_path in supplied_models:
                self.model_path = os.path.join(FfmpegQualityMetrics.DEFAULT_VMAF_MODEL_DIRECTORY, self.model_path)
            else:
                raise FfmpegQualityMetricsError(
                    f"Could not find model at {self.model_path}. Please set --model-path to a valid VMAF .json model file."
                )

        try:
            if self.verbose:
                print_info(
                    f"Writing temporary VMAF information to: {self.temp_files['vmaf']}"
                )

            vmaf_opts = {
                "model_path": win_path_check(self.model_path),
                "phone_model": "1" if phone_model else "0",
                "log_path": win_path_check(self.temp_files["vmaf"]),
                "log_fmt": "json",
                "psnr": "1",
                "ssim": "1",
                "ms_ssim": "1",
                "n_threads": str(self.n_threads),
            }

            vmaf_opts_string = ":".join(f"{k}={v}" for k, v in vmaf_opts.items())

            filter_chains = [
                f"[1][0]scale2ref=flags={self.scaling_algorithm}[dist][ref]",
                "[dist]setpts=PTS-STARTPTS[dist1]",
                "[ref]setpts=PTS-STARTPTS[ref1]",
                f"[dist1][ref1]libvmaf='{vmaf_opts_string}'",
            ]

            self._run_ffmpeg_command(filter_chains, desc="VMAF")

            if not self.dry_run:
                with open(self.temp_files["vmaf"], "r") as in_vmaf:
                    vmaf_log = json.load(in_vmaf)
                    for frame_data in vmaf_log["frames"]:
                        # append frame number, increase +1
                        frame_data["metrics"]["n"] = int(frame_data["frameNum"]) + 1
                        self.vmaf_data.append(frame_data["metrics"])

        except Exception as e:
            raise e
        finally:
            if os.path.isfile(self.temp_files["vmaf"]):
                os.remove(self.temp_files["vmaf"])

        return self.vmaf_data

    def _run_ffmpeg_command(self, filter_chains=[], desc=""):
        """
        Run the ffmpeg command to get the quality metrics.
        The filter chains must be specified manually.
        'desc' can be a human readable description for the progress bar.
        """
        if not self.framerate:
            ref_framerate, dist_framerate = self._get_framerates()
        else:
            ref_framerate = self.framerate
            dist_framerate = self.framerate

        cmd = [
            "ffmpeg",
            "-nostdin",
            "-y",
            "-threads",
            str(self.threads),
            "-r",
            str(ref_framerate),
            "-i",
            self.ref,
            "-r",
            str(dist_framerate),
            "-i",
            self.dist,
            "-filter_complex",
            ";".join(filter_chains),
            "-an",
            "-f",
            "null",
            NUL,
        ]

        if self.progress:
            ff = FfmpegProgress(cmd)
            with tqdm(total=100, position=1, desc=desc) as pbar:
                for progress in ff.run_command_with_progress():
                    pbar.update(progress - pbar.n)
        else:
            run_command(cmd, self.dry_run, self.verbose)

    def calc_ssim_psnr(self):
        """Calculate SSIM and PSNR

        Raises:
            e: A generic error

        Returns:
            dict: SSIM and PSNR results, each with their own key
        """
        try:
            if self.verbose:
                print_info(
                    f"Writing temporary SSIM information to: {self.temp_files['ssim']}"
                )
                print_info(
                    f"Writing temporary PSNR information to: {self.temp_files['psnr']}"
                )

            filter_chains = [
                f"[1][0]scale2ref=flags={self.scaling_algorithm}[dist][ref]",
                "[dist]setpts=PTS-STARTPTS[distpts]",
                "[ref]setpts=PTS-STARTPTS[refpts]",
                "[distpts]split[dist1][dist2]",
                "[refpts]split[ref1][ref2]",
                f"[dist1][ref1]psnr='{win_path_check(self.temp_files['psnr'])}'",
                f"[dist2][ref2]ssim='{win_path_check(self.temp_files['ssim'])}'",
            ]

            self._run_ffmpeg_command(filter_chains, desc="PSNR and SSIM")

            if not self.dry_run:
                with open(self.temp_files["psnr"], "r") as in_psnr:
                    # n:1 mse_avg:529.52 mse_y:887.00 mse_u:233.33 mse_v:468.25 psnr_avg:20.89 psnr_y:18.65 psnr_u:24.45 psnr_v:21.43
                    lines = in_psnr.readlines()
                    for line in lines:
                        line = line.strip()
                        fields = line.split(" ")
                        frame_data = {}
                        for field in fields:
                            k, v = field.split(":")
                            frame_data[k] = round(float(v), 3) if k != "n" else int(v)
                        self.psnr_data.append(frame_data)

                with open(self.temp_files["ssim"], "r") as in_ssim:
                    # n:1 Y:0.937213 U:0.961733 V:0.945788 All:0.948245 (12.860441)\n
                    lines = in_ssim.readlines()
                    for line in lines:
                        line = line.strip().split(" (")[0]  # remove excess
                        fields = line.split(" ")
                        frame_data = {}
                        for field in fields:
                            k, v = field.split(":")
                            if k != "n":
                                # make psnr and ssim keys the same
                                k = "ssim_" + k.lower()
                                k = k.replace("all", "avg")
                            frame_data[k] = round(float(v), 3) if k != "n" else int(v)
                        self.ssim_data.append(frame_data)

        except Exception as e:
            raise e
        finally:
            if os.path.isfile(self.temp_files["psnr"]):
                os.remove(self.temp_files["psnr"])
            if os.path.isfile(self.temp_files["ssim"]):
                os.remove(self.temp_files["ssim"])

        return {"ssim": self.ssim_data, "psnr": self.psnr_data}

    @staticmethod
    def get_brewed_vmaf_model_path():
        """
        Hack to get path for VMAF model from Linuxbrew

        Returns:
            str: the path
        """
        stdout, _ = run_command(["brew", "--prefix", "libvmaf"])
        cellar_path = stdout.strip()

        model_path = os.path.join(cellar_path, "share", "model")

        return model_path

    @staticmethod
    def get_default_vmaf_model_path():
        """
        Return the default model path depending on whether the user is running Homebrew
        or has a static build.

        Returns:
            str: the path
        """
        if has_brew() and ffmpeg_is_from_brew():
            # If the user installed ffmpeg using homebrew
            return os.path.join(
                # FIXME: change this once VMAF 2.0 is bundled with homebrew!
                FfmpegQualityMetrics.get_brewed_vmaf_model_path(),
                "vmaf_v0.6.1.pkl",
            )
        else:
            # return the bundled file as a fallback
            return os.path.join(
                os.path.dirname(__file__), "vmaf_models", "vmaf_v0.6.1.json"
            )

    @staticmethod
    def get_supplied_vmaf_models():
        """
        Return a list of VMAF models supplied with the software.

        Returns:
            list: A list of VMAF model names
        """
        return os.listdir(FfmpegQualityMetrics.DEFAULT_VMAF_MODEL_DIRECTORY)

    def get_global_stats(self):
        """
        Return a dictionary for each calculated metric, with different statstics

        Returns:
            dict: A dictionary with stats
        """
        for key in ["ssim", "psnr", "vmaf"]:
            data = getattr(self, key + "_data")
            if len(data) == 0:
                continue

            value_key = key if key == "vmaf" else key + "_avg"
            values = [float(entry[value_key]) for entry in data]
            self.global_stats[key] = {
                "average": round(np.average(values), 3),
                "median": round(np.median(values), 3),
                "stdev": round(np.std(values), 3),
                "min": round(np.min(values), 3),
                "max": round(np.max(values), 3),
            }

        return self.global_stats

    def get_results_csv(self) -> str:
        """
        Return a CSV string with the data

        Returns:
            str: The CSV string
        """
        all_dfs = []

        if self.vmaf_data:
            all_dfs.append(pd.DataFrame(self.vmaf_data))

        if self.psnr_data and self.ssim_data:
            all_dfs.append(pd.DataFrame(self.psnr_data))
            all_dfs.append(pd.DataFrame(self.ssim_data))

        if not all_dfs:
            raise FfmpegQualityMetricsError("No data calculated!")

        try:
            df = reduce(lambda x, y: pd.merge(x, y, on="n"), all_dfs)

            df["input_file_dist"] = self.dist
            df["input_file_ref"] = self.ref

            cols = df.columns.tolist()
            cols.insert(0, cols.pop(cols.index("n")))
            df = df.reindex(columns=cols)
            return df.to_csv(index=False)
        except Exception as e:
            raise FfmpegQualityMetricsError(f"Error merging data to CSV: {e}")

    def get_results_json(self) -> str:
        """
        Return the results as JSON string

        Returns:
            str: The JSON string
        """
        ret = {}
        for key in ["ssim", "psnr", "vmaf"]:
            data = getattr(self, key + "_data")
            if len(data) == 0:
                continue
            ret[key] = data
        ret["global"] = self.get_global_stats()
        ret["input_file_dist"] = self.dist
        ret["input_file_ref"] = self.ref

        return json.dumps(ret, indent=4)



# Always prefer setuptools over distutils
from setuptools import setup
# To use a consistent encoding
from codecs import open
import os

here = os.path.abspath(os.path.dirname(__file__))

# Versioning
with open(os.path.join(here, 'ffmpeg_quality_metrics', '__init__.py')) as version_file:
    version = eval(version_file.read().split("\n")[2].split("=")[1].strip())

# Get the long description from the README file
with open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='ffmpeg_quality_metrics',
    version=version,
    description='Calculate quality metrics with FFmpeg (SSIM, PSNR, VMAF)',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/slhck/ffmpeg-quality-metrics',
    author='Werner Robitza',
    author_email='werner.robitza@gmail.com',
    license='MIT',
    install_requires=['pandas', 'tqdm', 'ffmpeg-progress-yield'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Topic :: Multimedia :: Video',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8'
    ],
    packages=['ffmpeg_quality_metrics'],
    include_package_data=True,
    package_data={
        'ffmpeg_quality_metrics': ['vmaf_models/*.json']
    },
    entry_points={
        'console_scripts': [
            'ffmpeg_quality_metrics=ffmpeg_quality_metrics.__main__:main',
        ],
    }
)
